# CLI Specific Event Names
EVENT_CLI_COMMAND_EXECUTED = "CLI execution succeeded"
EVENT_CLI_COMMAND_FAILED = "CLI execution failed"

# CLI Specific Property Names
PROP_COMMAND_NAME = "command_name"
PROP_CLI_VERSION = "cli_version"
PROP_ORG_ID = "org_id"
PROP_PROJECT_ID = "project_id"
